//------------------------------------------------------------------*
//  TBrdcast component for delphi 3    v1.0                         *
//  Jean-Pierre Linteau   (ram.software@sympatico.ca)               *
//                                                                  *
//  This component is used to easily broadcast (send) and receive   *
//  message to/from other applications. Messages contains 3 parts : *
//  Received shortstring, received double and received key.         *
//------------------------------------------------------------------*
//  Properties:                                                     *
//    InKey : The value of the key received   (integer)             *
//    InData : double received                                      *
//    InText : Shortstring received                                 *
//    Key : The value of the key to send  (integer)                 *
//    OutText : the shortstring to send                             *
//    OutData : the data to send                                    *
//    EnableInput : enable reception of message                     *
//    EnableOutput : enable sending of message                      *
//                                                                  *
//  Method :                                                        *
//    Broadcast : This will send the stuff  to other apps.          *
//                                                                  *
//  Events :                                                        *
//    OnBroadcastMsg : fire when a message is received              *
//                                                                  *
//  Note:                                                           *
//  The key value (an integer) could be use to "secure" communica-  *
//  between processes. Setting the receiver to verify the value     *
//  transmitted.                                                    *
//------------------------------------------------------------------*
//  Copyright 1997 : Jean-Pierre Linteau                            *
//                                                                  *
//  LEGAL : This component is FREEWARE. I make no guarantees,       *
//  or warranties of any kind, you USE IT AT YOUR OWN RISK.         *
//                                                                  *
//  Can be freely used abused and distributed in commercial and     *
//  private environments, provided this notice is not modified in   *
//  any way.                                                        *
//------------------------------------------------------------------*
//  History :                                                       *
//                                                                  *
//  v 1.0  : Initial release  (april 98 )                           *
//                                                                  *
//  wish list : replacing shortstring with string...                *
//                                                                  *
//------------------------------------------------------------------*

unit Brdcast;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  Tlisten = procedure(inText: string ; inData : double ; inKey :integer)
          of object;

  TMsgPack = record
   texte : shortstring;
   data  : double;
             end;

  TBroadcast = class(TComponent)

  private
    FOnBroadcastMsg : Tlisten;
    FWinHandle : HWND;
    FKey : DWORD;
    FenableOutput, FenableInput : boolean;
    FOutText : shortstring;
    FOutData : double;
    t : TcopyDataStruct;
    dataPack, InMsg : ^TMsgPack;
    precv : pcopydataStruct;
  protected
    Constructor Create(AOwner: TComponent); Override;
    Destructor  Destroy; Override;
    procedure WndProc(var Msg : Tmessage);
  public
    InText : string;
    InData : double;
    InKey : integer;
    procedure Broadcast;
  published
    property Key : DWORD read Fkey write Fkey;
    property EnableInput : boolean read FEnableInput write FEnableInput;
    property EnableOutput : boolean read FEnableOutput write FEnableOutput;
    property OutText : shortString read FOutText write FOutText;
    property OutData : double read FOutData write FOutData;
    property OnBroadcastMsg: Tlisten read fOnBroadcastMsg write fOnBroadcastMsg;
  end;

procedure Register;

implementation

Constructor TBroadcast.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);
     FWinHandle  := AllocateHWND(WndProc);
end;

procedure TBroadcast.Broadcast;
begin
if FenableOutput then
  begin
    new(datapack);
    datapack^.texte:= FOutText;
    datapack^.data:= FOutData;
    t.dwData:= Fkey;
    t.lpData:= datapack;
    t.cbData:= sizeOf(datapack^);
    SendMessage(wnd_broadcast,WM_COPYDATA,FWinHandle,LongInt(@t));
  end;
end;

procedure TBroadcast.WndProc(var msg: TMessage);
begin
  if (enableInput AND (msg.msg = WM_COPYDATA)) then
     begin
     precv := pCopyDataStruct(msg.Lparam);
     inMsg:= precv.lpdata;
     inText:= inMsg^.texte;
     inData:= inMsg^.data;
     inKEY:= precv.dwData;
     if assigned(FonBroadcastMsg) then FonBroadcastMsg(inText,inData,inkey);
     end;
//msg.Result:= DefWindowProc(FwinHandle,msg.msg,msg.wParam,Msg.lparam);
end;

Destructor TBroadcast.Destroy;
begin
     DeallocateHWnd(FWinHandle);
     inherited Destroy;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TBroadcast]);
end;

end.
